#!/usr/bin/env python

######################################################
# Process mup input file for midi/printing/viewing
# Bob van der Poel <bvdp@xplornet.com>, Oct 1999.
# Added -f and -B options, Jan, 2000
# Added -g and -d options, August, 2001
# I honestly couldn't figure out the shell stuff anymore...so
# Converted to python, August, 2001
# Use getopt.gnu_getops() May/2005
# preview program option Nov/2011

import os
import sys
import commands
import getopt

Version = 1.3

# set the program to use for screen mode
#VIEWER = "gv -spartan"
VIEWER = "evince"

#################################################

class Opts:


    def __init__(self):
        """ Option parser. """

        self.out = "screen"
        self.copies = 1
        self.outfile = "/tmp/MupTemp.%s" % os.getpid()
        self.defines = ""
        self.infiles = []

        sopts = "bBTegptmc:f:d:vP"
        lopts = ["bflat", "bass", "tab", "eflat", "gbass", "print", "test",
                     "midi", "copies", "file=", "define=", "version", "pdf"]
        try:
            opts, args = getopt.gnu_getopt(sys.argv[1:], sopts, lopts )

        except getopt.GetoptError:
            self.usage()

        for o,a in opts:

            if o in ('-b', '--bflat'):
                self.defines +=" -DBFLAT"

            elif o in ('-B', '--bass'):
                self.defines += " -DBASS"

            elif o in ('-T', '--tab'):
                self.defines += " -DTAB"

            elif o in ('-e', '--eflat'):
                self.defines += " -DEFLAT"

            elif o in ('-g', '--gbass'):
                self.defines += " -DGBASS"

            elif o in ('-p', '--print'):
                self.out = "printer"

            elif o in ('-c', '--copies'):
                self.copies = int(a)

            elif o in ('-t', '--test'):
                self.out = "test"

            elif o in ('-m', '--midi'):
                self.out = "midi"

            elif o in ('-f', '--file'):
                self.out = 'file'
                self.outfile = a

            elif o in ('-d', '--define'):
                self.defines += " -D%s" % a

            elif o in ('-P', '--pdf'):
                self.out = "pdf"
                
            elif o in ('-v', '--version'):
                print Version
                sys.exit(0)


        self.infiles = args



    def usage(self, message=''):
        print "mp - process/view mup file."
        print "  usage: mp [opts] file"
        print "  options:"
        print "    -b, --bflat            define DBFLAT for B-flat (tenor)"
        print "    -B, --bass             define DBASS for C-bass clef (melody)"
        print "    -e, --eflat            define DEFLAT for E-flat (alto)"
        print "    -g, --gbass            define DGBASS for bass guitar"
        print "    -T, --tab              define TAB for tab chart"
        print "    -d, --define           define additional define"
        print "    -p, --print            output to printer"
        print "    -c N, --copies=N       print N copies"
        print "    -t, --test             test only, display log file"

        print "    -m, --midi             create midi file"
        print "    -P  --pdf              create PDF file"
        print "    -f<FILE>, --file=FILE  print to <FILE>"
        print "    -v, --version          print version number"

        if message > '':
            print "   ", message
            print

        sys.exit(1)


##########################################################################

# Start of program

# Parse args, get filename

opts = Opts()

if len(opts.infiles) <1:
	print "You must specify a mup input file"
	sys.exit(1)

# If we have 2 args and one is a emacs backup file, process the non-backup

elif len(opts.infiles) == 2:
    opts.infiles.sort()
    if opts.infiles[0]+"~" == opts.infiles[1]:
        infile = opts.infiles[0]

elif len(opts.infiles) == 1:
    infile = opts.infiles[0]

else:
    print "You must specify only one input file"
    sys.exit(1)




#########################################
# Process the file, either to PS or MIDI


if opts.out == "midi":
    sts, output = commands.getstatusoutput( "mup %s %s -m %s.mid" %
              ( infile, opts.defines, infile ))

else:
    std, output = commands.getstatusoutput( "mup %s %s >%s" %
                ( infile, opts.defines, opts.outfile ))


print output   # The result of the MUP command is echoed to the screen


#########################################
# We should have a valid midi or ps file.
# But, parse the mup log first. If any lines
# start with 'error:' we terminate.


for a in output.split('\n'):
    if a.startswith('! Error:'):
        try:
            os.remove(opts.outfile)
        except:  # the output file might not exist
            pass
        sys.exit(1)


###################################################
# Now display the file, or whatever.

if opts.out== 'screen':
	sts=None
	try:
		sts, output = commands.getstatusoutput("%s %s" % (VIEWER, opts.outfile ))
	except KeyboardInterrupt:
		pass
	except:
		print "Unexpected error ... there may be scratch files in /tmp"
		sys.exit(1)

	if sts:
		print "Unable to execute display command: %s" % output
	try:
		os.remove(opts.outfile)
	except:
		print "Could not delete scratch file '%s'" % opts.outfile


elif opts.out== 'printer':

    if opts.copies == 1:
        print "Printing 1 copy of '%s'" % infile
    else:
        print "Printing %s copies of '%s'" % (opts.copies, infile)

    sts, output = commands.getstatusoutput("lpr -#%s %s" % (opts.copies, opts.outfile) )
    if sts:
        print "Unable to execute print command: %s" % opts.outfile

    try:
        os.remove(opts.outfile)
    except:
	print "Could not delete scratch file '%s'" % opts.outfile


elif opts.out == 'midi':
    print "Midi file '%s.mid' generated" % infile

elif opts.out == 'pdf':
    
    # Convert complete/path/name/foo.mup --> foo.pdf
    pdfFile = os.path.basename(infile).replace(".mup", "") + ".pdf"

    sts, output = commands.getstatusoutput("ps2pdf %s %s" % (opts.outfile, pdfFile) )
    if sts:
        print "Unable to convert '%s' to PDF." % opts.outfile
    else:
        print "PDF file '%s' has been created." % pdfFile

    try:
        os.remove(opts.outfile)
    except:
        print "Could not delete scratch file '%s'" % opts.outfile

elif opts.out == 'file':
    print "Postscript file '%s' has been created" % opts.outfile


elif opts.out == 'test':
    sys.exit(0)


else:
    print "Unknown output mode . . . FIXME!"

